#include <xc.h>
#include "lcd.h"
#include "bmp\img1.h"
#include "bmp\img2.h"
#include "bmp\img3.h"
#include "bmp\img4.h"
#include "bmp\img6.h"
#include "bmp\img7.h"
#include "fonts\fnt68.h"
#include "fonts\fnt88.h"
#include "fonts\fnt88b.h"
#include "fonts\fnt816.h"
#define _XTAL_FREQ 10000000
#define LCD_RS		LATA1//RA1
#define LCD_RW		LATA2//RA2
#define LCD_EN		LATA3//RA3
#define LCD_CS_B	LATA4//RA4
#define LCD_RST		LATA5//RA5
#define BUS			PORTD
#define BUS_DIR		TRISD
#define LCD_M_DIR	TRISA
#define BUS_IN		255
#define BUS_OUT		0
#define LCD_WIDTH	160
#define LCD_HEIGHT	80
#define LCD_CHAR	20
#define LCD_STRING	10


//==============================================================================
// lcd_Strobe
//==============================================================================
#define lcd_strobe() __delay_us(1); LCD_EN = 1; __delay_us(1); LCD_EN = 0;__delay_us(1);
//==============================================================================
// lcd_Busy
//==============================================================================
void
lcd_busy()
 	{
  	unsigned char stat;
  		LCD_CS_B = 0;
  		BUS_DIR = BUS_IN;                    // read busy flag from HB61830
  		LCD_RS = 1;
  		LCD_RW = 1;
   		do
    		{
     			LCD_EN = 1;
     			__delay_us (3);
     			stat      = BUS;     			
     			LCD_EN = 0;
     			__delay_us (3);
    		}
   		while (stat & 0x80);                // == 0x80 (i.e, busy flag)
  		LCD_CS_B = 1;
 }
//==============================================================================
// Put data (send one char to LCD)
//==============================================================================
void lcd_putdata(unsigned char reg, unsigned char data)
 	{
  		lcd_busy();
  		LCD_CS_B = 0;                       // HD61830 on
  		BUS_DIR  = BUS_OUT;
  		LCD_RS    = 1;
  		LCD_RW    = 0;
  		BUS    = reg;
  		lcd_strobe();
  		LCD_RS    = 0;
  		LCD_RW    = 0;
  		BUS    = data;
  		lcd_strobe();
  		LCD_CS_B = 1;                       // HD61830 off
  		BUS_DIR  = BUS_IN;                  // avoid short-circuit when BUS is shared
	}
//==============================================================================
// lcd_reset
//==============================================================================
void lcd_reset()
	{
		BUS_DIR = BUS_OUT;
		BUS = 0;
		BUS_DIR  = BUS_IN;
		ADCON1=0b00000110;
		LCD_M_DIR=BUS_OUT;
		LCD_RS = 0;
		LCD_RW = 0;
		LCD_EN = 0;
		LCD_CS_B = 0;
		LCD_RST = 0;
		__delay_ms (3);
		LCD_RST = 1;
		__delay_ms (3);		
	}
//==============================================================================
// Initialization text mode
//==============================================================================
void lcd_init_text()
	{
  		lcd_putdata(SET_LCD_MODE, 0x3C);    // text mode (MODE CONTROL COMMAND)
  		lcd_putdata(SET_CH_PITCH, 0x75);    // native 5x7 font (CHARACTER PITCH CMD)
  		lcd_putdata(SET_CHAR_NUM, 20-1);    // 40 chars per line
  		lcd_putdata(SET_TIME_DIVS,80);    // duty ratio 64 (128 do not work!)
  		lcd_putdata(SET_CURS_POS, 7);       // position cursor
  		lcd_putdata(SET_HOM_L_ADDR, 0);
  		lcd_putdata(SET_HOM_H_ADDR, 0);
  		lcd_putdata(SET_CUR_L_ADDR, 0);
  		lcd_putdata(SET_CUR_H_ADDR, 0);
  		lcd_mode = LCD_TEXT;
	 }
//==============================================================================
// Initialization graph mode
//==============================================================================
void lcd_init_graphic()
 	{
  		lcd_putdata(SET_LCD_MODE, 0x32);    // graphic mode (MODE CONTROL COMMAND)
  		lcd_putdata(SET_CH_PITCH, 0x77);    // 8 bits per byte
  		lcd_putdata(SET_CHAR_NUM, 20-1);    // 30 bytes per line (address 0 to 29)
  		lcd_putdata(SET_TIME_DIVS,80);    // duty ratio 64 (128 do not work!)
  		lcd_putdata(SET_HOM_L_ADDR, 0);
  		lcd_putdata(SET_HOM_H_ADDR, 0);
  		lcd_putdata(SET_CUR_L_ADDR, 0);
  		lcd_putdata(SET_CUR_H_ADDR, 0);
  		lcd_mode = LCD_GRAPH;
  		lcd_graph_font = FNT68;
	 }
//==============================================================================
// Goto x,y (text & graph modes)
//==============================================================================
void lcd_goto(unsigned char x, unsigned char y)
 	{
  	unsigned int n;
  	if (lcd_mode == LCD_GRAPH)
   		{
    		n=(y*LCD_WIDTH+x)/8;                      // graph mode
   		}
  	else                                 // (lcd_mode == LCD_TEXT)
   		{
    		n=20*y+x;                           // text mode
   		}
  	lcd_putdata(SET_CUR_L_ADDR,n&0xff);
  	lcd_putdata(SET_CUR_H_ADDR,n>>8);
 }
//==============================================================================
// Byte burst prepare
//==============================================================================
void lcd_byte_burst_prepare()
 	{
  		lcd_busy();
  		LCD_CS_B = 0;
  		BUS_DIR  = BUS_OUT;
  		LCD_RS    = 1;
  		LCD_RW    = 0;
  		BUS    = LCD_WR_DATA;
  		lcd_strobe();
 	}
//==============================================================================
// Byte burst plot
//==============================================================================
void lcd_byte_burst_plot(unsigned char xyz)
 	{
  		BUS_DIR  = BUS_OUT;
  		LCD_RS   = 0;
  		LCD_RW   = 0;
  		LCD_CS_B = 0;
  		BUS = xyz;
  		lcd_strobe();
  		lcd_busy();
 	}
//==============================================================================
// Set plot
//==============================================================================
void lcd_set_plot(unsigned char x, unsigned char y, unsigned char onOff)
 	{
  	unsigned char pixel;
  		pixel=(y*LCD_WIDTH+x)%8;
  		lcd_goto(x,y);
  		if (onOff == 0)
   			{
    			lcd_putdata(LCD_CLEAR_BIT, pixel);
   			}
  		else
   			{
    			lcd_putdata(LCD_SET_BIT, pixel);
   			}
 	} 	
//==============================================================================
// Put string (graph & text modes)
//==============================================================================
void lcd_puts(const unsigned char * s, unsigned char x_ref, unsigned char y_ref)
	{
 	unsigned char vplt;
 	const unsigned char *s2;
 	unsigned char fit_begin, aux, size;
 	unsigned char pxplt;
 	unsigned char fnt_height, fnt_width;
 	const unsigned char *fnt;
 	unsigned int data_byte, data_byte_aux;
  		if (lcd_mode == LCD_TEXT)            // HD61830 native text mode: uses
   			{                                   // internal fonts. Disavantages:
    			while(*s) lcd_putdata(LCD_WR_DATA, *s++);                         // just 2 font sizes! Because of that,this mode is almost not usefull
   			}
 		else
   			{
    			switch(lcd_graph_font)             // in graphical mode, we do generate
     				{                                 // our own fonts!
      					case FNT68:
       						{
        						fnt_height = FNT68_HEIGHT;
        						fnt_width  = FNT68_WIDTH;      // 1-byte wide for most fonts
        						fnt        = &fnt68[0];
        						break;
       						}
      					case FNT88:
       						{
        						fnt_height = FNT88_HEIGHT;
        						fnt_width  = FNT88_WIDTH;
        						fnt        = &fnt88[0];
        						break;
       						}
      					case FNT88B:
       						{
        						fnt_height = FNT88B_HEIGHT;
                				fnt_width  = FNT88B_WIDTH;
        						fnt        = &fnt88b[0];
        						break;
       						}
      					case FNT816:
       						{
        						fnt_height = FNT816_HEIGHT;
        						fnt_width  = FNT816_WIDTH;
        						fnt        = &fnt816[0];
        						break;
       						};
     				} // switch
    			// *PART A* (see function header)
    			// Plots "out of a byte" (misaligned) pixels. Starts with the left-upper
    			// pixel of a misaligned character
    			fit_begin = (x_ref % 8);
    			if (fit_begin)
     				{
      					data_byte = (*s - 32)*fnt_height;
      					for (vplt = 0; vplt < fnt_height; vplt++)
       						{
        						for (pxplt = 0; ( pxplt < (8-fit_begin) ); pxplt++)
         							{
          								aux = (fnt[data_byte+vplt]) & (0b00000001 << pxplt);
          								lcd_set_plot(x_ref+pxplt,y_ref+vplt, aux);
         							}
       						}
      					x_ref = x_ref + (8-fit_begin);  // Aligned adress, i.e., the misaligned bits were plotted up to the specified
     				};                               // fnt_height
    			// *PART B* (see function header)
    			// Plots horizontal bytes first. Takes advantage of the burst mode to
    			// speed up the plotting
    			s2 = s;                                        // vplot = no. of pixels
    			for (vplt = 0; vplt < fnt_height; vplt++)      // Prepare burst write mode
     				{                                             // Plots "line-by-line" of
      					s = s2;                                      // the string of chars.
      					size = 0;                                    // It takes advantage of the
      					lcd_goto(x_ref,y_ref+vplt);                  // burst plot, which reduces
      					lcd_byte_burst_prepare();                    // drastically the amount of
      					while(*s)                                    // access to the LCD
        					{                                          // (remember: accesses takes
         						data_byte = (*s++ - 32)*fnt_height + vplt;// to much time:6uSeconds/ea)
         						if (fit_begin)                            // -That's why It's better
          							{                                        // reading the string
           								if (*s)                                 // "fnt_height x S" times
            								{                                      // than access the LCD
             									data_byte_aux = (*s - 32)*fnt_height + vplt; // twice
             									aux = ( fnt[data_byte]     >> (8-fit_begin) ) | \
                   								( fnt[data_byte_aux] << (fit_begin  ) );
             									lcd_byte_burst_plot(aux);
             									size = size + fnt_width;
            								};
          							}
         						else                                      // best case: x_ref is
          							{                                        // coincident with 0,8,16
           								lcd_byte_burst_plot(fnt[data_byte]);    // and so on
          							};
        					} // while
     				} // for
    			// *PART C* (see function header)
    			// Plots "out of a byte" (misaligned) pixels in the end of *s
    			if (fit_begin)
     				{
       					*s--;
       					x_ref = x_ref + size;                       // cursor where it should be,
       					data_byte = (*s - 32)*fnt_height;           // i.e, = ref + (8-fit_begin)
       					for (vplt = 0; vplt < fnt_height; vplt++)   // + no. of byte burst to be
        					{                                          // plot
         						for (pxplt = 0; ( pxplt < fit_begin ); pxplt++)
          							{
           								aux = (fnt[data_byte+vplt]) & (0b10000000 >> (fit_begin-pxplt-1) );
           								lcd_set_plot(x_ref+pxplt,y_ref+vplt, aux);
          							}
        					}
     				};
   			} // else
 	}
//==============================================================================
// Bitmap plot
//==============================================================================
void lcd_bitmap_plot(unsigned char x00, unsigned char y00, \
                unsigned char img_width, unsigned char img_height, \
                const unsigned char * image)
 	{
  	unsigned char vplot;
  	unsigned char p_plot;
  	unsigned int aux_plot;
  	unsigned char aux_plot_2;
  	unsigned char f_begin, i_width;
  	unsigned char img_size;
	  	// *PART AA*
  		// Plots "out of a byte" (misaligned) pixels. Starts with the left-upper
  		f_begin = (x00 % 8);
  		i_width = (img_width%8);
  		if (f_begin)
   			{
    			aux_plot = 0;
    			for ( vplot = 0; vplot < img_height; vplot++ )
     				{
      					for (p_plot = 0; ( p_plot < (8-f_begin) ); p_plot++)
       						{
        						aux_plot_2 = image[aux_plot] & (0b00000001 << p_plot);
        						lcd_set_plot(x00+p_plot,y00+vplot,aux_plot_2);
       						}
      					aux_plot = aux_plot + img_width/8;
      					if (i_width) aux_plot++;
     				}
    			x00 = x00 + (8-f_begin);        // Aligned adress, i.e., the misaligned bits were plotted up to the specified fnt_height
   			};                               
  		// *PART BB*
  		// Plots horizontal bytes first. Takes advantage of the burst mode to
  		// speed up the plotting
  		if (f_begin)
   			{
     			for (vplot = 0; vplot < img_height; vplot++)
      				{
       					img_size = 8;
       					lcd_goto(x00,y00+vplot);
       					lcd_byte_burst_prepare();
       					while(img_width > img_size)
        					{
         						aux_plot = *image++;
         						aux_plot_2 = *image;
         						img_size = img_size + 8;
         						aux_plot = ( aux_plot   >> (8-f_begin) ) | \
                    			( aux_plot_2 << (f_begin  ) );
         						lcd_byte_burst_plot(aux_plot);
        					}
       					*image++;
      				}
     			img_size = img_size - 8;
   			}
  		else if (i_width)
   			{
     			for (vplot = 0; vplot < img_height; vplot++)
      				{
       					img_size = 8;
       					lcd_goto(x00,y00+vplot);
       					lcd_byte_burst_prepare();
       					while(img_width > img_size)
         					{
          						aux_plot = *image++;
          						img_size = img_size + 8;
          						lcd_byte_burst_plot(aux_plot);
         					}
       					*image++;
      				}
    			img_size = img_size - 8;
   			}
  		else
   			{
     			for (vplot = 0; vplot < img_height; vplot++)
      				{
       					img_size = 0;
       					lcd_goto(x00,y00+vplot);
       					lcd_byte_burst_prepare();
       					while(img_width > img_size)
         					{
          						aux_plot = *image++;
          						img_size = img_size + 8;
          						lcd_byte_burst_plot(aux_plot);
         					}
      				}
   			}
  		// *PART CC*
  		// Plots "out of a byte" (misaligned) pixels in the end of *s
  		//if (img_width > img_size)
  		if (f_begin)
   			{
    			x00 = x00 + img_size;
    			image--;
    			for (vplot = img_height; vplot > 0; vplot-- )
     				{
    					for (p_plot = 0; ( (img_size+p_plot) < (img_width-(8-f_begin)); p_plot++)
       						{
        						aux_plot_2 = aux_plot & (0b10000000 >> (f_begin-p_plot-1));
        						lcd_set_plot(x00+p_plot,y00+vplot-1,aux_plot_2);
       						}
      					image = image - (img_width/8);
      					if (i_width) *image--;
      					aux_plot = *image;
     				}
   			}
  		else if (i_width)
   			{
    			x00 = x00 + img_size;
    			image--;
    			for (vplot = img_height; vplot > 0; vplot-- )
     				{
      					for (p_plot = 0; ( (img_size+p_plot) < img_width); p_plot++)
       						{
        						aux_plot_2 = aux_plot & (0b10000000 >> (7-p_plot));
        						lcd_set_plot(x00+p_plot,y00+vplot-1,aux_plot_2);
      						}
      					image = image - (img_width/8);
      					if (i_width) *image--;
      					aux_plot = *image;
     				}
   			};
	}
//==============================================================================
// LCD clear
//==============================================================================
void lcd_clear(void)
 	{
  	int i;
  		lcd_putdata(SET_CUR_L_ADDR,0);
  		lcd_putdata(SET_CUR_H_ADDR,0);
  		if (lcd_mode == LCD_TEXT) for(i=0;i<(LCD_CHAR*LCD_STRING);i++) lcd_putdata(LCD_WR_DATA,0); // clear screen (text mode) 16*40 = 640 char
  		else if (lcd_mode == LCD_GRAPH) for(i=0;i<((LCD_WIDTH*LCD_HEIGHT)/8);i++) lcd_putdata(LCD_WR_DATA,0); // clear screen (graph mode) // 240*128/8 = 3840 pixels
  		lcd_putdata(SET_CUR_L_ADDR,0);
  		lcd_putdata(SET_CUR_H_ADDR,0);
 	};
//==============================================================================
// LCD ON/OFF     LCD     1:ON  0:OFF
//==============================================================================
void lcd_disp_on_off(unsigned char disp)
 	{
  		if (lcd_mode == LCD_TEXT)            // mode text
   			{
    			lcd_putdata(SET_LCD_MODE, 0x1C | (disp ? 0x30 : 0x00));
   			}
  		else                                 // mode graph
   			{
    			lcd_putdata(SET_LCD_MODE, 0x12 | (disp ? 0x30 : 0x00));
   			}
 	}








/*

/*


/*******************************************************************************
  for (vplot = 0; vplot < img_height; vplot++)   // V: vplot = no. of pixels
   {
     lcd_goto(x00,vplot+y00);
     lcd_byte_burst_prepare();                   // prepare burst write mode
     for (hplot = 0; hplot < img_width; hplot++) // H: hplot=(no. of pixels)/8
      {
        lcd_byte_burst_plot(*image++);           // remember to "prepare burst
      };                                         // write mode" before entering
    };                                           // burst_plot!
*******************************************************************************/
/* 


//==============================================================================
// Bitmap inverter
//
//
// same offset (x00,y00) restrictions from lcd_bitmap_plot (see above).
//
// reads a full line (burst reading) and save it in data_inv. Then return
// the lcd cursor to the beginning of the same line and start to re-write it
// but inverting the value of the bits.
//
//==============================================================================
void
lcd_graph_inv(unsigned char x00, unsigned char y00, \
              unsigned char img_width, unsigned char img_height)
 {
  unsigned char vplot, hplot;
  unsigned char data_inv[30];                    // 30bytes = 240pixels / 8bits
  unsigned char index;

  if ( !(img_width % 8) )
    img_width = img_width >> 3;
  else
    img_width = (img_width >> 3) + 1;

  index = 0;
  for (vplot = 0; vplot < img_height; vplot++)   //  V: vplot = no. of pixels
   {
     lcd_goto(x00,vplot+y00);                    // --+
      lcd_busy();                                //   |
      BUS_DIR  = BUS_OUT;                        //   |
      LCD_CS_B = 0;                              //   |
      LCD_RS    = 1;                             //   |
      LCD_RW    = 0;                             //   |
      BUS    = LCD_RD_DATA;                      //   |  LCD data read burst
      lcd_strobe();                              //   |
      BUS_DIR = BUS_IN;                          //   |
      LCD_RS = 0;                                //   |
      LCD_RW = 1;                                //   |
      lcd_strobe();                              //   |<-- this is a dummy read
      lcd_busy();                                // --+
     for (hplot = 0; hplot < img_width; hplot++) // H: hplot=(no. of pixels)/8
      {
        //BUS_DIR  = BUS_IN;                     // already IN due to previous
        LCD_CS_B = 0;                            // "lcd_busy"
        LCD_RS = 0;
        LCD_EN = 1;
        asm("nop");
        //*data_inv++ = BUS;
        data_inv[hplot] = BUS;
        asm("nop");
        LCD_EN = 0;
        lcd_busy();
      };
     lcd_goto(x00,vplot+y00);                    // carrier return (same line)
     lcd_byte_burst_prepare();                   // prepare burst write mode
     for (hplot = 0; hplot < img_width; hplot++)
      {
        //lcd_byte_burst_plot(~*data_inv++);     //////////////////////////////
        lcd_byte_burst_plot(~data_inv[hplot]);   // remember to "prepare burst
      };                                         // write mode" before entering
   };                                            // burst_plot!
}

void
lcd_line(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
 {
  unsigned char i,dx,dy,px,py;
  unsigned int x,y;

  if (x2 == x1)                        // incl. VERTICAL
   {
    for(py=y1;py<y2;py++)
    {
     lcd_set_plot(x1, py, 1);
    }
    lcd_set_plot(x2, y2, 1);           // last pixel
   }
  else if (y2 == y1)                   // incl. HORIZONTAL
   {
    for(px=x1;px<x2;px++)
    {
     lcd_set_plot(px, y1, 1);
    }
    lcd_set_plot(x2, y2, 1);           // last pixel
   }
  else if (y2 > y1)                    // incl. POSITIVE
   {
    dx=x2-x1;
    dy=y2-y1;
    px=x1;
    py=y1;
    lcd_set_plot(x1, y1, 1);           // first pixel
    if (dx>=dy)                        // incl. is more horizontal than vertical
    {
      y = y1;
      for(i=0;i<dx;i++)
      {
       y+=dy;
       if (y>=dx)
        {
         y-=dx;
         py+=1;
        }
       px+=1;
       lcd_set_plot(px, py, 1);
      }
    }
    else                               // incl. is more vertical than horizontal
    {
      x = x1;
     for(i=0;i<dy;i++)
      {
       x+=dx;
       if (x>=dy)
        {
         x-=dy;
         px+=1;
        }
       py+=1;
       lcd_set_plot(px, py, 1);        //??????
      }
    };
   }
  else if (y2 < y1)                    // incl. NEGATIVE
   {
    dx=x2-x1;
    dy=y1-y2;                          // inverted to produce a positive value
    dx=dx;
    dy=dy;
    px=x1;
    py=y2;                             // Note: due to inverted "dy",
    lcd_set_plot(x2, y2, 1);           // the first pixel became the last one
    if (dx>=dy)                        // incl. is more horizontal than vertical
    {
      y = y1;
      py=y1;
      for(i=0;i<dx;i++)
      {
       y+=dy;
       if (y>=dx)
        {
         y-=dx;
         py-=1;
        }
       px+=1;
       lcd_set_plot(px, py, 1);
      }
    }
    else                               // incl. is more vertical than horizontal
    {
      x = x1;
      px=x2;
     for(i=0;i<dy;i++)
      {
       x+=dx;
       if (x>=dy)
        {
         x-=dy;
         px-=1;
        }
       py+=1;
       lcd_set_plot(px, py, 1);
      }
    };
   }
 }


//==============================================================================
// Horizontal line
//==============================================================================
void
lcd_line_H(unsigned char xref, unsigned char ypas, \
           unsigned char y1, unsigned char y2)
 {
  unsigned char y;
  for(y=y1; y<=y2; y += ypas)
    {
     lcd_set_plot(xref, y, 1);
    }
 }


//==============================================================================
// Vertical line
//==============================================================================
void
lcd_line_V(unsigned char yref, unsigned char xpas, \
           unsigned char x1, unsigned char x2)
 {
  unsigned char x;
  for(x=x1; x<=x2; x+=xpas)
   {
    lcd_set_plot(x, yref, 1);
   }
 }


//==============================================================================
// Grid
//==============================================================================
void
lcd_grid(void)
 {
  signed char tmpY, pasY;
  //unsigned char x;                   // lcd_line_V + H
  for(tmpY=84; tmpY>=20; tmpY-=8) {    // 92 12 -8
    if (tmpY!=52) pasY = 8;
    else        pasY = 4;
    lcd_line_V(tmpY, pasY, 0, 63);     // 10 lignes
    //for(x=0; x<=63; x+=pasY){
    //  lcd_plot(x, tmpY, 1);
    //}
  }
  lcd_line_H(32, 4, 12, 92);           // Xref, pasY, y1, y2
  //for(x=12; x<=92; x += 4){          //y
  //  lcd_plot(32, x, 1);              //y
  //}
  ////lcd_rect(0,12, 63, 92);
  lcd_line_H(0, 1, 12, 92);
  lcd_line_H(63, 1, 12, 92);
  lcd_line_V(12, 1, 0, 63);
  lcd_line_V(92, 1, 0, 63);
 }


//==============================================================================
// Rectangle
//==============================================================================
void
lcd_rect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
 {
  lcd_line_H(x1, 1, y1, y2);
  lcd_line_H(x2, 1, y1, y2);
  lcd_line_V(y1, 1, x1, x2);
  lcd_line_V(y2, 1, x1, x2);
 }


//==============================================================================
// Circle
//                             Center(x,y) + radius
//==============================================================================
void
lcd_circle(unsigned char cx, unsigned char cy, unsigned char radius)
 {
  signed int tswitch;
  unsigned char y, x = 0;
  unsigned char d;

  d = cy - cx;
  y = radius;
  tswitch = 3 - 2 * radius;
  while (x <= y)
    {
     lcd_set_plot(cx + x,    cy + y, 1);
     lcd_set_plot(cx + x,    cy - y, 1);
     lcd_set_plot(cx - x,    cy + y, 1);
     lcd_set_plot(cx - x,    cy - y, 1);
     lcd_set_plot(cy + y - d, cy + x, 1);
     lcd_set_plot(cy + y - d, cy - x, 1);
     lcd_set_plot(cy - y - d, cy + x, 1);
     lcd_set_plot(cy - y - d, cy - x, 1);
     if (tswitch < 0) tswitch += (4 * x + 6);
       else
        {
          tswitch += (4 * (x - y) + 10);
          y--;
        }
     x++;
    }
 }






//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================

/*
//=========================================================================================
// ecrit un nombre en hexa sur le LCD
//=========================================================================================
void
lcd_hexa(unsigned char c){
  lcd_putchar( (c >> 4) ); //hexa2ascii
  lcd_putchar( (c & 0x0F) ); //hexa2ascii
}

//=========================================================================================
// lcd_uchar : affiche un unsigned char
//=========================================================================================
void
lcd_uchar(const char *str, unsigned char ligne, unsigned char col, unsigned char val){
  unsigned char tmp;
  ///###lcd_string_loc_lc(str, ligne, col);
  lcd_goto(ligne, col);
  lcd_puts(str);

  tmp = val;
  lcd_putchar(val/100 + 0x30);
  val = val%100;
  lcd_putchar(val/10 + 0x30);
  lcd_putchar(val%10 + 0x30);
}
*/

/*********************************************************
* graphics support function
*********************************************************/
/*
void lcg4pixels(unsigned char x,unsigned char y,unsigned char xc,unsigned char yc,unsigned char c)
{
  lcd_set_plot(xc+x,yc+y,c);
  lcd_set_plot(xc-x,yc+y,c);
  lcd_set_plot(xc+x,yc-y,c);
  lcd_set_plot(xc-x,yc-y,c);
}
*/
/*********************************************************
* draw an ellipse
* Bresenham's ellipse algorithm
*********************************************************/
/*
void lcgellipse(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, unsigned char c)
//void lcgellipse(unsigned char xc,unsigned char yc,unsigned char a0,unsigned char b0,unsigned char c)
//void drawline2d(int x0, int y0, int x1, int y1, int color)
{
       unsigned char i;
       unsigned char steep = 1;
       unsigned char sx, sy;  // step positive or negative (1 or -1)
       unsigned char dx, dy;  // delta (difference in X and Y between points)
       unsigned char e;

        // inline swap. On some architectures, the XOR trick may be faster

       unsigned char tmpswap;
#define SWAP(a,b) tmpswap = a; a = b; b = tmpswap;

        // optimize for vertical and horizontal lines here

       //dx = abs(x1 - x0);
       dx = (x1 - x0);
       sx = ((x1 - x0) > 0) ? 1 : -1;
       //dy = abs(y1 - y0);
       dy = (y1 - y0);
       sy = ((y1 - y0) > 0) ? 1 : -1;
       if (dy > dx) {
               steep = 0;
               SWAP(x0, y0);
               SWAP(dx, dy);
               SWAP(sx, sy);
       }
       e = (dy << 1) - dx;
       for (i = 0; i < dx; i++) {
               if (steep) {
                       //plot(x0,y0,color);
                       lcd_set_plot(x0,y0,c);

               } else {
                       //plot(y0,x0,color);
                       lcd_set_plot(y0,x0,c);
               }
               while (e >= 0) {
                       y0 += sy;
                       e -= (dx << 1);
               }
               x0 += sx;
               e += (dy << 1);
       }
}
*/

/*
void lcgellipse(unsigned char xc,unsigned char yc,unsigned char a0,unsigned char b0,unsigned char c)
{
  unsigned char x,y;
  float a,b;
  float asqr,twoasqr;
  float bsqr,twobsqr;
  float d,dx,dy;

  x=0;
  y=b0;
  a=a0;
  b=b0;
  asqr=a*a;
  twoasqr=2*asqr;
  bsqr=b*b;
  twobsqr=2*bsqr;

  d=bsqr-asqr*b+asqr/4;
  dx=0;
  dy=twoasqr*b;
  while(dx<dy){
    lcg4pixels(x,y,xc,yc,c);
    if(d>0){
      y--;
      dy-=twoasqr;
      d-=dy;
    }
    x++;
    dx+=twobsqr;
    d=d+bsqr+dx;
  }
  d=d+(3*(asqr-bsqr)/2-(dx+dy))/2;
  while(y>=0){
    lcg4pixels(x,y,xc,yc,c);
    if(d<0){
      x++;
      dx+=twobsqr;
      d+=dx;
    }
    y--;
    dy-=twoasqr;
    d=d+asqr-dy;
  }
}
*/
